@@ -2,12 +2,18 @@
 #===== statistic functions ==================
 
 sub abs {
+	
+	check_prototype(@_, '$');
+	
 	$_[0] < 0 ? -$_[0] : $_[0];
 }
 
 # usage: sign( [SCALAR] )
 # return: SCALAR
 sub sign {
+	
+	check_prototype(@_, '$');
+	
 	$_[0] > 0 ? 1 : $_[0] < 0 ? -1 : 0;
 }
 
@@ -15,6 +21,9 @@ sub sign {
 # return: SCALAR
 # description: 
 sub sum {
+	
+	check_prototype(@_, '\@');
+	
     my $array = shift;
     
     my $sum = 0;
@@ -27,29 +36,37 @@ sub sum {
 
 # usage: mean( [ARRAY REF] )
 # return: SCALAR
-# description: ֵ
 sub mean {
+	
+	check_prototype(@_, '\@');
+	
     return sum($_[0]) / len($_[0]);
 }
 
 # usage: geometric_mean( [ARRAY REF] )
 # return: SCALAR
-# description: ƽ
 sub geometric_mean {
+	
+	check_prototype(@_, '\@');
+	
     return exp(sum(sapply($_[0], sub{log($_[0])})) / len($_[0]));
 }
 
 # usage: sd( [ARRAY REF] )
 # return: SCALAR
-# description: ׼
 sub sd {
+	
+	check_prototype(@_, '\@$?');
+	
     return sqrt(var(@_));
 }
 
 # usage: var( [ARRAY REF], [SCALAR] )
 # return: SCALAR
-# description: 
 sub var {
+	
+	check_prototype(@_, '\@$?');
+	
     my $array = shift;
     my $mean = shift;
 	
@@ -66,8 +83,10 @@ sub var {
 
 # usage: cov( [ARRAY REF], [ARRAY REF] )
 # return: SCALAR
-# description: Э
 sub cov{
+	
+	check_prototype(@_, '\@\@');
+	
     my $array1 = shift;
     my $array2 = shift;
 	
@@ -83,8 +102,10 @@ sub cov{
 
 # usage: cor( [ARRAY REF], [ARRAY REF] )
 # return: SCALAR
-# description: ϵ
 sub cor {
+	
+	check_prototype(@_, '\@\@$?');
+	
     my $array1 = shift;
     my $array2 = shift;
 	my $method = shift || "pearson";
@@ -99,8 +120,10 @@ sub cor {
 
 # usage: dist( [ARRAY REF], [ARRAY REF], [SCALAR] )
 # return: SCALAR
-# description: 
 sub dist {
+	
+	check_prototype(@_, '\@\@$?');
+	
 	my $vector1 = shift;
 	my $vector2 = shift;
 	my $method = shift || "euclidean";
@@ -127,8 +150,10 @@ sub dist {
 
 # usage: freq( [ARRAY REF], [ARRAY REF], ... )
 # return: HASH REF
-# description: ÿֵƵ,Ƕ
 sub freq {
+	
+	check_prototype(@_, '(\@)+');
+	
     my @category = @_;
     
     my $f = {};
@@ -141,13 +166,18 @@ sub freq {
 
 # same as frequency
 sub table {
+	
+	check_prototype(@_, '(\@)+');
+	
 	return freq(@_);
 }
 
 # usage: scale( [ARRAY REF], "zvalue|percentage|sphere")
 # return: HASH REF
-# description: һ
 sub scale {
+	
+	check_prototype(@_, '\@$?');
+	
     my $array = shift;
 	my $type = shift || "zvalue";
 	$type = lc($type);
@@ -158,7 +188,7 @@ sub scale {
 		
 		return sapply($array, sub { ($_[0] - $min)/($max - $min) });
 	}
-	elsif($type eq "sphere") {    # all the points are on the surface of the super sphere
+	elsif($type eq "sphere") {    # all the points are on the surface of the unit super sphere
 		my $s = sqrt(sum(sapply($array, sub{$_[0]**2})));
 		
 		return sapply($array, sub { $_[0]/$s });
@@ -173,9 +203,10 @@ sub scale {
 
 # usage: sample( [ARRAY REF], [SCALAR], "p" => [ARRAY REF], "replace" => 0|1 )
 # return: ARRAY REF
-# description: г趨ppеĸʳ飬pĳҪarrayĳ
-#              replaceʾǷǷŻس
 sub sample {
+	
+	check_prototype(@_, '\@($|\@|\%)+');
+	
     my $array = shift;
     my $size = shift;
 	
@@ -183,7 +214,7 @@ sub sample {
 	             "replace" => 0,
 	             @_ };
 	my $sum_p = sum($setup->{"p"});
-    my $p = sapply($setup->{"p"}, sub {$_[0]/$sum_p}); # pֵ01֮
+    my $p = sapply($setup->{"p"}, sub {$_[0]/$sum_p}); #0-1
     my $replace = $setup->{"replace"};
     
 	if(!$replace and len($array) < $size) {
@@ -195,15 +226,15 @@ sub sample {
 	}
 	
     my $sample = [];
-    my $ecdf = _ecdf($p);  # մۼӸ
+    my $ecdf = _ecdf($p);  
     
-    if($replace) {  # Żس
+    if($replace) { 
         for(my $i = 0; $i < $size; $i ++) {
             my $ind = _get_index_from_p(rand(), $ecdf);
             push(@$sample, $array->[$ind]);
         }
     }
-    else {          # Żسÿ
+    else {
         my $array_copy;
         push(@$array_copy, @$array);
         my $p_copy;
@@ -227,7 +258,6 @@ sub sample {
 
 # usage: ecdf( [ARRAY REF])
 # return: ARRAY REF
-# description: ۻ
 sub _ecdf {
     my $p = shift;
     my $ecdf = cumf($p, \&sum);
@@ -236,7 +266,7 @@ sub _ecdf {
 
 # usage: _get_index_from_p( [SCALAR], [ARRAY REF])
 # return: SCALAR
-# description: ۼƸȷɵĸ
+# [, )
 sub _get_index_from_p {
     my $r = shift;
     my $p = shift;
@@ -255,8 +285,10 @@ sub _get_index_from_p {
 
 # usage: rnorm( [SCALAR], [SCALAR], [SCALAR])
 # return: ARRAY REF
-# description: ֲ̬
 sub rnorm {
+	
+	check_prototype(@_, '${1,3}');
+	
     my $size = shift;
     my $mean = shift;
     my $sd = shift;
@@ -284,8 +316,10 @@ sub rnorm {
 
 # usage: rbinom( [SCALAR], [SCALAR])
 # return: ARRAY REF
-# description: ɶʽֲ1ɹ0ʧܣpɹĸʣ
 sub rbinom {
+	
+	check_prototype(@_, '$+');
+	
 	my $size = shift;
 	my $p = shift;
 	
@@ -298,8 +332,10 @@ sub rbinom {
 
 # usage: max( [ARRAY REF] )
 # return: SCALAR
-# description: ֵ
 sub max {
+	
+	check_prototype(@_, '\@');
+	
     my $array = shift;
     my $max = $array->[0];
     for (@$array) {
@@ -310,8 +346,10 @@ sub max {
 
 # usage: min( [ARRAY REF] )
 # return: SCALAR
-# description: Сֵ
 sub min {
+	
+	check_prototype(@_, '\@');
+	
     my $array = shift;
     my $min = $array->[0];
     for (@$array) {
@@ -322,8 +360,10 @@ sub min {
 
 # usage: which_max( [ARRAY REF] )
 # return: SCALAR
-# description: ֵ±꣨һ
 sub which_max {
+	
+	check_prototype(@_, '\@');
+	
     my $array = shift;
     my $max = $array->[0];
 	my $which_max = 0;
@@ -337,8 +377,10 @@ sub which_max {
 
 # usage: which_min( [ARRAY REF] )
 # return: SCALAR
-# description: Сֵ±꣨һ
 sub which_min {
+	
+	check_prototype(@_, '\@');
+	
     my $array = shift;
     my $min = $array->[0];
 	my $which_min = 0;
@@ -352,8 +394,10 @@ sub which_min {
 
 # usage: median( [ARRAY REF] )
 # return: SCALAR
-# description: ֵ
 sub median {
+	
+	check_prototype(@_, '\@');
+	
     my $array = shift;
     
     my $sort = sort_array($array);
@@ -370,9 +414,10 @@ sub median {
 
 # usage: quantile( [ARRAY REF], [SCALAR | ARRAY REF] )
 # return: SCALAR | ARRAY REF
-# description: λ㣬pһ򷵻һpһ򷵻
-#              λ01֮Ե
 sub quantile {
+	
+	check_prototype(@_, '\@($|\@)?');
+	
     my $array = shift;
 	my $p = shift;
 	$p = defined($p) ? $p : [0, 0.25, 0.5, 0.75, 1];
@@ -409,8 +454,10 @@ sub quantile {
 
 # usage: iqr( [ARRAY REF])
 # return: SCALAR
-# description: 75λ25λĲֵ
 sub iqr {
+	
+	check_prototype(@_, '\@');
+	
 	my $array = shift;
 	
 	my $q = quantile($array, [.25, .75]);
@@ -418,6 +465,9 @@ sub iqr {
 }
 
 sub cumf {
+	
+	check_prototype(@_, '\@(\&)?');
+	
 	my $array = shift;
 	my $function = shift || sub {my $x = $_[0]; $x->[$#$x];};
 	
